;' DKFS-51 (C) Dmitriy Korabelnikov 2005-2008.
;' This software DKFS-51 is a disk operating system
;' for 8-bit and 16-bit microcontrollers based on I2C/SPI/SD-Card FLASH memory.
;' DKFS-51 is intended promarily for MCS-51 family of microcontrollers
;' and Keil C51 compiler but can be also used for other 8 and 16-bit
;' microcontrollers (like AVR, ATMEGA) assuming rewritten asm, interrupt
;' and hardware code (like timers and serial port inicialization).
;' In the current state this is a COMMERCIAL software.
;' Please contact with author by e-mail d_korabelnikov@mail.ru
;' if you want to use this code in your devices or applications.
;' Next text is Russian Win-1251
;' DKFS-51 (C)   2005-2008.
;' DKFS-51      8  16-
;'    I2C/SPI/SD-Card FLASH .
;' DKFS-51     MCS-51  
;'  Keil C51    
;'   8  16-  (AVR,ATMEGA)  
;'   ,     
;' (      ).
;'      .    
;'  e-mail d_korabelnikov@mail.ru
;'           .
?C?LIB_CODE	SEGMENT		CODE
		RSEG	?C?LIB_CODE
public clrxram
clrxram:mov dptr,#0
cxram1:	clr a
	movx @DPTR,a
	inc dptr
	mov a,dph
	orl a,dpl
	jnz cxram1
	ret


public _swaplong

_swaplong:
	mov a,R7
	xch a,R4
	mov R7,a
	mov a,R6
	xch a,R5
	mov R6,a
	ret

public _swapint
_swapint:
	mov a,R7
	xch a,R6
	mov R7,a
	ret


; Clock support

SDA bit P3.5
SCL bit P1.3

call_iwait macro
	endm


wrc:       push acc
wrc1:      call cl_reset
	   call cl_start
	   mov  a,#0D0h
	   call cl_write
	   jc    wrc1
	   mov  a,b
	   call cl_write
	   pop   acc
	   call cl_write
	   call cl_stop
	   ret

	

public 	_ClkSetTime
_ClkSetTime:
	  mov DPH,R2
	  mov DPL,R1
	  mov b,#0
	  movx a,@DPTR; mov a,MSec
	  inc DPTR
	  call wrc
	  mov b,#1
	  movx a,@DPTR; mov a,MMin
	  inc DPTR
	  call wrc
	  mov b,#2
	  movx a,@DPTR; mov a,Mhour
	  call wrc
	  ret

public 	_ClkSetDate
_ClkSetDate:
	  mov DPH,R2
	  mov DPL,R1
	  inc DPTR ; skip sec,min,hr
	  inc DPTR
	  inc DPTR
	  mov b,#4
	  movx a,@DPTR ; mov a,MDate
	  inc DPTR
	  call wrc
	  mov b,#5
	  movx a,@DPTR ; mov a,MMonth
	  inc DPTR
	  call wrc
	  mov b,#6
	  movx a,@DPTR ; mov a,MYearL
	  call wrc
	  ret


cl_reset:call cl_stop
	call_iwait
	call_iwait
	ret

public cl_start
cl_start:setb SDA
	call_iwait
	setb SCL
	call_iwait
	clr SDA
	call_iwait
	clr SCL
	call_iwait
	ret 

public cl_stop
cl_stop:clr  SCL
	call_iwait
	clr  SDA
	call_iwait
	setb SCL
	call_iwait
	setb SDA
	call_iwait
	call_iwait
	mov C,SDA
;	jnc cl_stop ; fix 2008 January 18 (command removed)
	ret 


public _wraddr
_wraddr:mov a,R6
	call cl_write
	jc clw0
	mov a,R7
	call cl_write
clw0:	ret

public _wrdata
_wrdata:mov a,R7

cl_write:clr SCL
	call_iwait
	push acc
	push B
	mov  B,#8
clw1:   add  a,acc
	mov  SDA,C
	call_iwait
	setb SCL
	call_iwait
	call_iwait
	clr SCL
	call_iwait
	djnz b,clw1
	pop B
	pop acc
	setb SDA
	call_iwait
	setb SCL
	call_iwait
	mov C,SDA
	call_iwait
	clr SCL
	call_iwait
	ret


public rdata
rdata:
cl_read:clr SCL
	setb SDA
	call_iwait
	push B
	clr  A
	mov  B,#8
clr1:   call_iwait
	setb SCL
	call_iwait
	mov   C,SDA
	addc a,acc
	call_iwait
	clr SCL
	call_iwait
	djnz b,clr1
	pop B
	clr SDA
	call_iwait
	setb SCL
	call_iwait
	call_iwait
	clr SCL
	call_iwait
	setb SDA
	mov R7,a
	ret

public _rdata2
_rdata2: mov DPH,R2
	mov DPL,R1
	; R4 - MSB, R5 LSB counter
rdata20:clr SCL
	setb SDA ; Use SDA for input
	mov a,R4
	orl a,R5
	jz rdata21
	clr  A
	rept 8
	setb SCL
	mov   C,SDA
	addc a,acc
	clr SCL
	endm

	clr SDA  ; fix 08 Jul 2005 ; zero pulse
	setb SCL
	clr SCL


	movx @DPTR,a
	inc DPTR
	dec R5
	cjne R5,#0FFh,rdata20
	dec R4
	jmp rdata20
rdata21:ret


if 0
public rdatanoa
rdatanoa:
cl_readnoa:clr SCL
	setb SDA
	call_iwait
	push B
	clr  A
	mov  B,#8
clr1noa:call_iwait
	setb SCL
	call_iwait
	mov   C,SDA
	addc a,acc
	call_iwait
	clr SCL
	call_iwait
	djnz b,clr1noa
	pop B
	setb SDA
	call_iwait
	setb SCL
	call_iwait
	call_iwait
	clr SCL
	call_iwait
	setb SDA
	mov R7,a
	ret
endif

public _clkrdtim
_clkrdtim: mov DPH,R2
	   mov DPL,R1
	   mov b,#0
	   call rdc1
	   movx @DPTR,a; mov MSec,a
	   inc DPTR
	   mov b,#1
	   call rdc2
	   movx @DPTR,a; mov MMin,a
	   inc DPTR
	   mov b,#2
	   call rdc2
	   movx @DPTR,a; mov MHour,a
	   inc DPTR
	   mov B,#3
	   call rdc2
	   nop   ; skip day of the week
	   mov b,#4
	   call rdc2
	   movx @DPTR,a; mov MDate,a
	   inc DPTR
	   mov b,#5
	   call rdc2
	   movx @DPTR,a; mov MMonth,a
	   inc DPTR
	   mov b,#6
	   call rdc2
	   movx @DPTR,a; mov MYearL,a
	   inc DPTR
	   call cl_stop
fixyear:; in progress - fix 1999/2000
	   mov a,#20h
	   movx @DPTR,a; mov MYEARH,#20h
	   ret


rdc1:      call cl_reset
	   call cl_start
	   mov  a,#0D0h
	   call cl_write
	   jc   rdc1   ; no answer
	   mov  a,b
	   call cl_write
	   call cl_start
	   mov  a,#0D1h
	   call cl_write
	   call cl_read
	   ret

rdc2:      call cl_read
	   ret

$include (crc16f.a51)


	end
