//' DKFS-51 (C) Dmitriy Korabelnikov 2005-2008.
//' This software DKFS-51 is a disk operating system
//' for 8-bit and 16-bit microcontrollers based on I2C/SPI/SD-Card FLASH memory.
//' DKFS-51 is intended promarily for MCS-51 family of microcontrollers
//' and Keil C51 compiler but can be also used for other 8 and 16-bit
//' microcontrollers (like AVR, ATMEGA) assuming rewritten asm, interrupt
//' and hardware code (like timers and serial port inicialization).
//' In the current state this is a COMMERCIAL software.
//' Please contact with author by e-mail d_korabelnikov@mail.ru
//' if you want to use this code in your devices or applications.
//' Next text is Russian Win-1251
//' DKFS-51 (C)   2005-2008.
//' DKFS-51      8  16-
//'    I2C/SPI/SD-Card FLASH .
//' DKFS-51     MCS-51  
//'  Keil C51    
//'   8  16-  (AVR,ATMEGA)  
//'   ,     
//' (      ).
//'      .    
//'  e-mail d_korabelnikov@mail.ru
//'           .
#define USEFAT8

#ifndef USEFAT8
#define FS_SHORT    unsigned int
#define FS_LONG     unsigned long
#define BAD_SECTOR  0xFFFE
#define END_SECTOR  0xFFFF
#else
#define FS_SHORT    unsigned char
#define FS_LONG     unsigned int
#define BAD_SECTOR  0xFE
#define END_SECTOR  0xFF
#endif

//    
#define F_NO_ROOM		8
//      (    )
#define F_NO_FILES      	18
//       - FLASH     
#define F_NOT_READY		21
//   
#define F_CRC_ERROR		23
//    ( 1   maxsector)
#define F_SECTOR_NOT_FOUND	27
//            
#define F_EOF			31
//  
#define F_NOTOPENED		32
//       (  f_create())
#define F_ALREADY_EXISTS        128

#ifndef maxdrive
#define maxdrive 1
xdata uns char diskdrive;
#endif

#define sectorsize  demosectorsize
#define sectorshift demosectorshift
#define maxsector   demomaxsector
#define sec_chip    maxsector

xdata unsigned char timebuf[10];
xdata uns char diskbuf[sectorsize+2];


FS_SHORT lastreadsector;

#ifndef USEFAT8
//     (   32 )
struct filerectype {uns char name[13];uns char attr;long time;FS_LONG size;FS_LONG maxsize;FS_SHORT cluster;uns long unused;};
#else
//     (   16 )
struct filerectype {uns char name[6];uns char attr;long time;FS_LONG size;FS_LONG maxsize;uns char cluster;};
#endif

#define dirsectorsize 1
xdata struct filerectype wdir;
#define maxfiles (sectorsize*dirsectorsize/sizeof(wdir))
xdata struct filerectype dir[maxfiles]; 

#ifndef USEFAT8
//     ,
//      fatwordsize >= maxsector+1
#define fatwordsize 256
FS_SHORT fat[fatwordsize];
#define fatsectorsize (2*fatwordsize/sectorsize)
#else
#define fatbytesize sectorsize
FS_SHORT fat[fatbytesize];
#define fatsectorsize (fatbytesize/sectorsize)
#endif

//    
#define firstdatasector (1+dirsectorsize+fatsectorsize)

//        
#ifndef USEFAT8
xdata uns char findbuf[14];
#else
xdata uns char findbuf[7];
#endif

uns char findrec,crec;bit copen;

//   f_seek, f_read, f_write
bit f_sectorvalid,f_writeop;

#define MOD_SEEK  0
#define MOD_READ  1
#define MOD_WRITE 2
//   
data char f_writemod;
data FS_SHORT acluster0,acluster,coffsHi,coffsHiN;
data uns int coffsLo,coffsLoN;
data FS_LONG loffset,lsize,lmaxsize;

// 
//      fat
FS_SHORT findfreesector();

// 
uns char secread(unsigned char *sectorbuf,FS_SHORT sector);

// 
uns char secwrite(unsigned char *sectorbuf,FS_SHORT sector);

bit dirvalid=0,fatvalid=0;

// 
uns char secformat(unsigned char *sectorbuf,FS_SHORT sector);

// 
uns char dirread();

// 
uns char dirwrite();

//  fat
uns char fatread();

// fat
uns char fatwrite();

//  
uns char diskformat(uns char drvnumber);

//   
uns char f_seek(FS_LONG pos);

//   
uns char f_read(uns char *buf,uns int nbytes);

//   
uns char f_write(uns char *buf,uns int nbytes);

//  
bit f_close();

//    
uns char f_exists(uns char *fname);


//    fname    maxisize
uns char f_create(uns char *fname,FS_LONG maxisize);

//  
bit f_open(uns char *fname);

// 
uns char f_delete(uns char *fname);

//  
bit f_setdrive(uns char drvnumber);

//    
#define f_start() (f_setdrive(1))

//     
#define f_stop() {if (copen) f_close();}

//     
void dircmd();

//     ( )
void dump(uns char *p);

//     ( )
void secdump(uns int clust);

