//' DKFS-51 (C) Dmitriy Korabelnikov 2005-2008.
//' This software DKFS-51 is a disk operating system
//' for 8-bit and 16-bit microcontrollers based on I2C/SPI/SD-Card FLASH memory.
//' DKFS-51 is intended promarily for MCS-51 family of microcontrollers
//' and Keil C51 compiler but can be also used for other 8 and 16-bit
//' microcontrollers (like AVR, ATMEGA) assuming rewritten asm, interrupt
//' and hardware code (like timers and serial port inicialization).
//' In the current state this is a COMMERCIAL software.
//' Please contact with author by e-mail d_korabelnikov@mail.ru
//' if you want to use this code in your devices or applications.
//' Next text is Russian Win-1251
//' DKFS-51 (C)   2005-2008.
//' DKFS-51      8  16-
//'    I2C/SPI/SD-Card FLASH .
//' DKFS-51     MCS-51  
//'  Keil C51    
//'   8  16-  (AVR,ATMEGA)  
//'   ,     
//' (      ).
//'      .    
//'  e-mail d_korabelnikov@mail.ru
//'           .

FS_SHORT findfreesector()
{FS_SHORT i;if (fat[0]!=firstdatasector) return 0; //     FAT
 for (i=firstdatasector;i<=maxsector;i++) {if (fat[i]==0) return i;} 
return 0;} 

uns char secread(unsigned char *sectorbuf,FS_SHORT sector)
{uns int flashaddr;
if ((sector>maxsector) || (sector==0)) return F_SECTOR_NOT_FOUND;
//ichip=(bit)((sector-1) / sec_chip);
flashaddr=(sectorsize+2)*(((uns int)sector-1) % sec_chip);
iread(sectorbuf,flashaddr,sectorsize+2);
if (crc16bad(sectorbuf,sectorsize)) return F_CRC_ERROR;
lastreadsector=sector;
return 0;}

uns char secwrite(unsigned char *sectorbuf,FS_SHORT sector)
{uns int flashaddr;uns int crc;
if ((sector>maxsector) || (sector==0)) return F_SECTOR_NOT_FOUND;
//ichip=(bit)((sector-1) / sec_chip);
flashaddr=(sectorsize+2)*(((uns int)sector-1) % sec_chip);
crc=crc16x(sectorbuf,sectorsize);*((uns int*)(sectorbuf+sectorsize))=crc;
iwrite(sectorbuf,flashaddr,sectorsize+2);
lastreadsector=sector;
return 0;}

uns char secformat(unsigned char *sectorbuf,FS_SHORT sector)
{uns char rbyte;
dirvalid=0,fatvalid=0;
memset(sectorbuf,0xFF,sectorsize);
rbyte=secwrite(sectorbuf,sector);if (rbyte) return rbyte;
rbyte= secread(sectorbuf,sector);if (rbyte) return rbyte;
memset(sectorbuf,0,sectorsize);
rbyte=secwrite(sectorbuf,sector);if (rbyte) return rbyte;
rbyte= secread(sectorbuf,sector);if (rbyte) return rbyte;
return 0;}

uns char dirread()
{uns char r;uns char *p;FS_SHORT sector;
if (dirvalid) return 0;
p=(uns char*)dir; for (sector=1;sector<=dirsectorsize;sector++) 
{r=secread(diskbuf,sector);if (r) return r;
memcpy(p,diskbuf,sectorsize);p+=sectorsize;}
dirvalid=1;return 0;
}

uns char dirwrite()
{uns char r;uns char *p;FS_SHORT sector;
p=(uns char*)dir; for (sector=1;sector<=dirsectorsize;sector++) 
{memcpy(diskbuf,p,sectorsize);r=secwrite(diskbuf,sector);if (r) {dirvalid=0;return r;}
 p+=sectorsize;}
return 0;
}

uns char fatread()
{uns char r;uns char *p;FS_SHORT sector;
if (fatvalid) return 0;
p=(uns char*)fat;
for (sector=1+dirsectorsize;sector<=dirsectorsize+fatsectorsize;sector++)
{lastreadsector=sector;r=secread(diskbuf,sector);if (r) return r;
memcpy(p,diskbuf,sectorsize);p+=sectorsize;}
fatvalid=1;return 0;
}

uns char fatwrite()
{uns char r;uns char *p;FS_SHORT sector;
p=(uns char*)fat;
for (sector=1+dirsectorsize;sector<=dirsectorsize+fatsectorsize;sector++)
{memcpy(diskbuf,p,sectorsize);
 lastreadsector=sector;r=secwrite(diskbuf,sector);if (r) {fatvalid=0;return r;}
p+=sectorsize;}
 return 0;
}

//  
uns char diskformat(uns char drvnumber)
{FS_SHORT itmp;uns char r,d;FS_SHORT free;bit badformat;
d=(drvnumber & 63) - 1;
//  
if (d>=maxdrive) return 0;
//    
if (copen) f_close();
//  
diskdrive=d;dirvalid=0;fatvalid=0;

printf("Formatting drive %c\n",diskdrive+'A');
badformat=0;
memset(dir,0,sizeof(wdir)*maxfiles);

#ifndef USEFAT8
memset(fat,0,fatwordsize*2);
#else
memset(fat,0,fatbytesize);
#endif

for (itmp=1;itmp<=maxsector;itmp++)
  {r=secformat(diskbuf,itmp); if (r) fat[itmp]=BAD_SECTOR;

#ifndef USEFAT8
  printf("sector %05d r=%02bx\r",itmp,r);}
#else
  printf("sector %03bu r=%02bx\r",itmp,r);}
#endif

printf("\n");

for (itmp=1;itmp<firstdatasector;itmp++)
#ifndef USEFAT8
	{if (fat[itmp]==BAD_SECTOR) {printf("Bad sector %05d (system area)\n",itmp);badformat=1;}}
#else
	{if (fat[itmp]==BAD_SECTOR) {printf("Bad sector %03bd (system area)\n",itmp);badformat=1;}}
#endif

fat[0]=firstdatasector;
dirwrite();fatwrite();

free=0;for (itmp=fat[0];itmp<=maxsector;itmp++) {if (fat[itmp]==0) free++;}

#ifndef USEFAT8
printf("\nFree sectors(bytes): %5d(%10ld)\n",free,sectorsize*(uns long)free);
#else
printf("\nFree sectors(bytes): %3bu(%5u)\n",free,sectorsize*(uns int)free);
#endif

if (badformat) {printf("Error formatting %c - (file operations are impossible)\n",diskdrive+'A');return 1;}
printf("Format %c complete\n",diskdrive+'A');
return 0;}

uns char f_seek(FS_LONG pos)
{data FS_SHORT tmpcluster,i;
if (copen==0) return F_NOTOPENED;
//       
if (pos>=lmaxsize) return F_EOF;

//     
if ((pos>=lsize) and (f_writemod!=MOD_WRITE)) return F_EOF;

//    ,    (  sectorsiz=2 POWER sectorshift)
coffsHiN=(FS_SHORT)(pos >> sectorshift);coffsLoN=(uns int)(pos & (sectorsize-1));

//       ,  =_
if (coffsHi==coffsHiN)
    tmpcluster=acluster;
//       fat (acluster0-  ) 
else 
    {if (fatvalid==0) fatread();
     tmpcluster=acluster0; for (i=0;i<coffsHiN;i++) {tmpcluster=fat[tmpcluster];
     if (!((tmpcluster>=firstdatasector) && (tmpcluster<=maxsector))) return F_EOF;}}

coffsHi=coffsHiN;coffsLo=coffsLoN;loffset=pos;

//         (  )
if (pos>lsize) lsize=pos;

//       (  )
if (tmpcluster==acluster) return 0;
// ...
//   ,   
if (f_writeop) {secwrite(diskbuf,acluster);f_writeop=0;}
//    (  )
f_sectorvalid=0;acluster=tmpcluster;
return 0;
}

uns char f_read(uns char *buf,uns int nbytes)
{data uns char r;
f_writemod=MOD_READ;
if (copen==0) return F_NOTOPENED;
while (nbytes)
{r=f_seek(loffset);if (r) return r;
//  (f_sectorvalid==0)    
if (f_sectorvalid==0) {f_sectorvalid=1;secread(diskbuf,acluster);}
*buf++=diskbuf[coffsLo];loffset++;nbytes--;}
return 0;
}

uns char f_write(uns char *buf,uns int nbytes)
{data uns char r;
f_writemod=MOD_WRITE;
if (copen==0) return F_NOTOPENED;
while (nbytes)
{r=f_seek(loffset);if (r) return r;
if (f_sectorvalid==0) {f_sectorvalid=1;secread(diskbuf,acluster);}
f_writeop=1;diskbuf[coffsLo]=*buf;buf++;loffset++;nbytes--;}
return 0;
}

bit f_close()
{
// ,    
if (copen==0) return 1;
//      dir  fat (     ),   
//  
if ((dirvalid==0) || (fatvalid==0)) {copen=0;return 1;}
//    ,   
if (f_writeop) {f_writeop=0;secwrite(diskbuf,acluster);}
//sprintf(prnbuf,"loffset %ld lsize %ld",loffset,lsize);prn(prnbuf);
//     
if ((loffset>lsize) and (loffset<=lmaxsize)) lsize=loffset;
wdir.size=lsize;
//    
// To do:     
//clkrdtim(timebuf);for (i=0;i<6;i++) timebuf[i]=froma(timebuf[i]);
wdir.time=usec(timebuf[5],timebuf[4],timebuf[3],timebuf[2],timebuf[1],timebuf[0]);

memcpy(&dir[findrec],&wdir,sizeof(wdir));
//   
dirwrite();
copen=0;
return 1;
}

uns char f_exists(uns char *fname)
{bit found;data uns char recn;uns char *p;
#ifndef USEFAT8
strncpy(findbuf,fname,13);findbuf[13]=0;
#else
strncpy(findbuf,fname,6);findbuf[6]=0;
#endif
for (p=findbuf;(*p);p++) *p=toupper(*p);
dirread();
found=0;recn=0;
while ((found==0) && (recn<maxfiles))
{if (strcmp(dir[recn].name,findbuf)==0) {findrec=recn;found=1;} else recn++;}
if (found) return recn+1;else return 0;
}

uns char f_create(uns char *fname,FS_LONG maxisize)
{bit found;data uns char recn;uns char *p;
long max; //  2^31
FS_SHORT freesec,prevsec;

//  -    
if (f_exists(fname)) return F_ALREADY_EXISTS;
if (copen!=0) f_close();
dirread();
found=0;recn=0;
for (recn=0;recn<maxfiles;recn++)
{memcpy(&wdir,&dir[recn],sizeof(wdir));
found=(wdir.name[0]<2);if (found) break;}
//  -      
if (found==0) return F_NO_FILES;
fatread();

#ifndef USEFAT8
strncpy(wdir.name,fname,12);
#else
strncpy(wdir.name,fname,5);
#endif
//      
p=wdir.name; while (*p) {*p=toupper(*p);p++;}
//date, time in progress
wdir.attr=0;wdir.time=0;
wdir.size=0L;
wdir.maxsize=maxisize;
//    
freesec=findfreesector();if (freesec==0) return F_NO_ROOM;

wdir.cluster=freesec;

#ifndef USEFAT8
wdir.unused=0L;
#endif

fat[freesec]=END_SECTOR; //     
//    1 
max=(long)maxisize;
if (max>sectorsize)
{max-=sectorsize; 
while (max>0L)
  {prevsec=freesec;freesec=findfreesector();if (freesec==0) return F_NO_ROOM;
   fat[prevsec]=freesec;fat[freesec]=END_SECTOR;
   max-=sectorsize;
   }
}
memcpy(&dir[recn],&wdir,sizeof(wdir));
//    ,  dir  fat   
dirwrite();
fatwrite();
return 0;
}

bit f_open(uns char *fname)
{
//      1 
if (copen!=0) f_close();
//   ,  0
if (f_exists(fname)==0) return 0;
dirread();fatread();
crec=findrec;
memcpy(&wdir,&dir[findrec],sizeof(wdir));
acluster0=wdir.cluster;
acluster=wdir.cluster;
lsize=wdir.size;
lmaxsize=wdir.maxsize;
f_sectorvalid=f_writeop=0;
coffsHi=(FS_SHORT)0;coffsLo=0;loffset=(FS_LONG)0;
crec=findrec;
f_writemod=MOD_SEEK;
copen=1;
return 1;
}

uns char f_delete(uns char *fname)
{uns char recn;
FS_SHORT sec,nextsec;

if (copen!=0) f_close();

recn=f_exists(fname);if (recn==0) return 0; // not found, in progress
recn--;
fatread();
//   fat
sec=dir[recn].cluster;
do {nextsec=fat[sec];fat[sec]=0;sec=nextsec;} while (sec!=END_SECTOR);

//    
memset(&dir[recn],0,sizeof(wdir));dir[recn].name[0]=1;
dirwrite();
fatwrite();
return 0;
}

bit f_setdrive(uns char drvnumber)
{uns char d;
d=(drvnumber & 63) - 1;
//  
if (d>=maxdrive) return 0;
//    
if (copen) f_close();
//  
diskdrive=d;dirvalid=0;fatvalid=0;
//    
if (dirread()) return 0;
//    fat
if (fatread()) return 0;
if (fat[0]!=firstdatasector) return 0;
return 1;
}

void dircmd()
{uns char recn;FS_SHORT free;
fatread();dirread();
printf("\nDrive %c:",diskdrive+'A');
#ifndef USEFAT8
printf("\nlen    |max len|YY-MM-DD HH:MM:SS|Clus|filename");
#else
printf("\nlen  |max l|YY-MM-DD HH:MM:SS|Clu|filename");
#endif
for (recn=0;recn<maxfiles;recn++) {if (dir[recn].name[0]>=32) {
// full info in progress
//       
btime(timebuf,dir[recn].time);
#ifndef USEFAT8
printf("\n%7ld %7ld %02bd-%02bd-%02bd %02bd:%02bd:%02bd %4u %s",\
dir[recn].size,dir[recn].maxsize,\
timebuf[5],timebuf[4],timebuf[3],timebuf[2],timebuf[1],timebuf[0],\
dir[recn].cluster,dir[recn].name);
#else
printf("\n%5u %5u %02bd-%02bd-%02bd %02bd:%02bd:%02bd %3bu %s",\
dir[recn].size,dir[recn].maxsize,\
timebuf[5],timebuf[4],timebuf[3],timebuf[2],timebuf[1],timebuf[0],\
dir[recn].cluster,dir[recn].name);
#endif
}}
//   
free=0;for (recn=fat[0];recn<=maxsector;recn++) {if (fat[recn]==0) free++;}
#ifndef USEFAT8
printf("\nFree sectors(bytes): %5d(%10ld)\n",free,sectorsize*(FS_LONG)free);
#else
printf("\nFree sectors(bytes): %3bu(%5u)\n",free,sectorsize*(FS_LONG)free);
#endif
}

void dump(uns char *p)
{uns char i,j,d;
for (j=0;j<sectorsize/16;j++)
{
for (i=0;i<16;i++) {printf("%02bX ",*p);p++;} printf("|");p-=16;
for (i=0;i<16;i++) {d=*p++;if (d<32) d='.';printf("%c",d);} printf("\n");
}
}

void secdump(uns int clust)
{f_close();secread(diskbuf,clust);dump(diskbuf);}

